unit cpEditInteger;
{
	v 1.0 : 30-08-1998
  Freeware component for Delphi 3 by Christian Poisson.
  Send any comments to cpoisson@mygale.org.

  TEdit components allowing only to enter integers with the possibility
  to use Roman numerals.
  A new property : Roman.
  There are 2 procedures (ToRoman and ToInt) to convert to or from Roman.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TcpEditInteger = class(TEdit)
  private
    { Dclarations prives }
    FRoman: boolean;
    function GetValue: Integer;
    procedure SetValue (NewValue: Integer);
    procedure SetRoman (NewValue: boolean);
		procedure KeyPress(var Key: Char);  override;
  protected
    { Dclarations protges }
    procedure Change; override;
  public
    { Dclarations publiques }
    constructor Create(AOwner: TComponent); override;
  published
    { Dclarations publies }
    property Value: Integer read GetValue write SetValue;
    property Roman: boolean read FRoman write SetRoman;
    function IntToRoman(nombre:integer): string;
    function RomanToInt(nombre:string): integer;
    procedure ToRoman;
    procedure ToInt;
  end;

procedure Register;

implementation

procedure TcpEditInteger.ToRoman;
begin
  Text:=Inttoroman(value);
	FRoman:=True;
end;

procedure TcpEditInteger.ToInt;
begin
  Text:=inttostr(value);
	FRoman:=false;
end;

function TcpEditInteger.RomanToInt(nombre:string): integer;
begin
	result:=0;
  while pos('M',nombre)=1 do
  	begin
      result:=result+1000;
      delete(nombre,1,1);
    end;
  if pos('CM',nombre)=1 then
  	begin
    	result:=result+900;
      delete(nombre,1,2);
    end;
  if pos('D',nombre)=1 then
  	begin
    	result:=result+500;
      delete(nombre,1,1);
    end;
  if pos('CD',nombre)=1 then
  	begin
    	result:=result+400;
      delete(nombre,1,2);
    end;
  while pos('C',nombre)=1 do
  	begin
      result:=result+100;
      delete(nombre,1,1);
    end;
  if pos('XC',nombre)=1 then
  	begin
    	result:=result+90;
      delete(nombre,1,2);
    end;
  if pos('L',nombre)=1 then
  	begin
    	result:=result+50;
      delete(nombre,1,1);
    end;
  if nombre='IL' then
  	begin
    	result:=result+49;
      nombre:='';
    end;
  if pos('XL',nombre)=1 then
  	begin
    	result:=result+40;
      delete(nombre,1,2);
    end;
  while pos('X',nombre)=1 do
  	begin
      result:=result+10;
      delete(nombre,1,1);
    end;
  if nombre='IX' then
  	begin
    	result:=result+9;
      nombre:='';
    end;
  if pos('V',nombre)=1 then
  	begin
    	result:=result+5;
      delete(nombre,1,1);
    end;
  if nombre='IV' then
  	begin
    	result:=result+4;
      nombre:='';
    end;
  while pos('I',nombre)=1 do
  	begin
      result:=result+1;
      delete(nombre,1,1);
    end;
  if nombre<>'' then result:=0;
end;

function TcpEditInteger.IntToRoman(nombre: integer): string;
begin
	result:='';
	while nombre>=1000 do
  	begin
    	result:=result+'M';
      nombre:=nombre-1000;
    end;
  if nombre>=900 then
  	begin
    	result:=result+'CM';
      nombre:=nombre-900;
    end;
	if nombre>=500 then
  	begin
    	result:=result+'D';
      nombre:=nombre-500;
    end;
	if nombre>=400 then
  	begin
    	result:=result+'CD';
      nombre:=nombre-400;
    end;
	while nombre>=100 do
  	begin
    	result:=result+'C';
      nombre:=nombre-100;
    end;
  if nombre>=90 then
  	begin
    	result:=result+'XC';
      nombre:=nombre-90;
    end;
	if nombre>=50 then
  	begin
    	result:=result+'L';
      nombre:=nombre-50;
    end;
	if nombre=49 then
  	begin
    	result:=result+'IL';
      nombre:=0;
    end;
	if nombre>=40 then
  	begin
    	result:=result+'XL';
      nombre:=nombre-40;
    end;
	while nombre>=10 do
  	begin
    	result:=result+'X';
      nombre:=nombre-10;
    end;
	if nombre=9 then
  	begin
    	result:=result+'IX';
      nombre:=0;
    end;
	if nombre>=5 then
  	begin
    	result:=result+'V';
      nombre:=nombre-5;
    end;
	if nombre=4 then
  	begin
    	result:=result+'IV';
      nombre:=0;
    end;
	while nombre>=1 do
  	begin
    	result:=result+'I';
      nombre:=nombre-1;
    end;
  FRoman:=true;
end;

constructor TcpEditInteger.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FRoman:=False;
 	text:='0';
end;

procedure TcpEditInteger.KeyPress(var Key: Char);
begin
	inherited;
  if FRoman then
  	begin
    	key:=upcase(key);
    	if not (key in ['I','V','X','L','C','D','M',#8]) then
  			begin
    			key:=#0;
      		beep;
    		end;
    end
  else if (not (key in ['0'..'9','-',#8])) or ((key='-') and (selstart<>0))  then
  	begin
    	key:=#0;
      beep;
    end;
end;

procedure TcpEditInteger.Change;
begin
	inherited;
  if not FRoman then
  	begin
		  if text='' then
  			begin
			  	text:='0';
    		  selstart:=0;
		      sellength:=1;
    		end
		  else if pos('--',text)=1 then
  			text:=copy(text,2,length(text));
    end;
end;

procedure TcpEditInteger.SetRoman(NewValue: boolean);
begin
  if NewValue<>FRoman then
  	begin
    	FRoman:=NewValue;
      if newvalue then text:=inttoroman(value);
    end;
end;

function TcpEditInteger.GetValue: Integer;
begin
	if FRoman then
  	Result:=romantoint(text)
  else
    Result:=StrToIntdef(Text,0);
end;

procedure TcpEditInteger.SetValue(NewValue: Integer);
begin
	if FRoman then
  	text:=inttoroman(newvalue)
  else
	  Text:=IntToStr(NewValue);
end;

procedure Register;
begin
  RegisterComponents('Standard', [TcpEditInteger]);
end;

end.
